from gtkeasyapp.easyprocess import *
from devicemanager.connectionmanager import *


##InternetConnectionProcess
##Manage internet connection
class ConnectionProcess(EasyProcess):
	##Constructor
	##@param processName : name of the process
	##@param ui : EasyWindow instance
	##@connectionName : name of internet connection
	##@param onConnected : callback when connected
	##@param onDisconnected : callback when disconnected
	def __init__(self, processName, ui, connectionName, onConnected, onDisconnected):
		#super constructor
		EasyProcess.__init__(self, processName, ui)
		
		#set members
		self._onConnected = onConnected
		self._onDisconnected = onDisconnected
		self.__connectionName = connectionName
		self.__connection = None
		self.__startup = True
		self._timeout = 60.0 #check connection every 60 seconds
		self.__connectedAtStartup = False
		self.__connections = ConnectionManager(self.__onConnected, self.__onDisconnected)
		
		#custom process
		self.custom(delayedTime=2.0)
	
			
	##Process : check connection
	def _process(self):
		#startup finished
		self.__startup = False

		#check connection
		if not self.__connections.isConnected():
			#init connection if necessary
			if self.__connection==None:
				self.__initConnection()
	
			#connect
			self.__connect()
		
			
	##Stopping process
	def _stopping(self):
		if self.__connections.isConnected() and not self.__connectedAtStartup:
			#disconnect
			self.__connections.disconnect()
	
	
	##Init internet connection
	##@return True if connection found successfully
	def __initConnection(self):
		#search data connexions
		connections = self.__connections.getConnections(ConnectionItem.CNX_TYPE_DATA, self.__connectionName)
		
		#get correct connection
		if len(connections)>0:
			self.__connection = connections[0]
			return True
		
		else:
			#no connection found
			return False
			
	
	##Disconnect from internet	
	def disconnect(self):
		self.__connections.disconnect()
			
			
	##Connect to internet
	def __connect(self):
		self.__connections.connectTo(self.__connection, self.__onConnected)
		
		
	##On connection established
	def __onConnected(self, *args):
		if self.__startup:
			#connected at startup
			
			#update flags
			self.__startup = False
			self.__connectedAtStartup = True
			
			#wait 1 second before calling back (to be sure all process are running)
			threading.Timer(1.0, self._onConnected).start()
		
		else:
			#just connected, callback
			self._onConnected()
	
	
	##On connection closed
	def __onDisconnected(self, *args):
		#callback
		self._onDisconnected()
		
		
		