import threading

##EasyProcess Exception class
class EasyProcessException(Exception):
	def __init__(self, message):
		Exception.__init__(self, message)

		

##Class that implement EasyWindow process
##An EasyProcess is an abstract class that permits easily create correct thread
##and is usable in GTK window
##@param ui : EasyWindow instance
class EasyProcess(threading.Thread):

	##Constructor
	##@param processName : name of the process
	##@param ui : EasyWindow instance
	def __init__(self, processName='', ui=None):
		#super constructor
		threading.Thread.__init__(self)
	
		#set members
		self._ui = ui
		self._stopThread = threading.Event() #stop flag
		self._timeout = 1.0 #by default thread execute process() every 1 second
		self._delayedTime = 0.0
		self._repeat = True
		self.__started = False
		
		#add myself to EasyWindow process list
		if len(processName)>0 and ui!=None:
			ui._addProcess(processName, self)
	
	
	##Run thread
	def run(self):
		#set started flag
		print ' --> "%s" started' % (self.__class__.__name__)
		self.__started = True
	
		#delay process start
		if self._delayedTime!=0.0:
			#delayed first process call
			self._stopThread.wait(self._delayedTime)
				
		#launch process
		if not self._repeat:
			#only once
			self._process()
			
		else:
			#until thread is stopped
			while not self._stopThread.isSet():
				#execute process
				self._process()
			
				#wait until timeout
				self._stopThread.wait(self._timeout)
				
		#stopping thread
		self._stopping()
		print ' --> "%s" stopped' % (self.__class__.__name__)

	
	##Stop thread
	def stop(self):
		#reset stop flag
		self._stopThread.set()
		
	
	##Configure custom behaviour
	##@param repeat : repeat or not thread after 1 execution
	##@param delayedTime : delayed first process execution
	def custom(self, repeat=True, delayedTime=0.0):
		self._repeat = repeat
		self._delayedTime = delayedTime
		
		
	##Return True if process already started
	def started(self):
		return self.__started
	
	
	##Stopping thread treatment
	def _stopping(self):
		raise EasyProcessException('_stopping() method must be implemented')
		
	
	##Main process method
	def _process(self):
		raise EasyProcessException('_process() method must be implemented')
	