import gobject
import pygtk
pygtk.require('2.0')
import gtk
import pango
import sys
import os
import platform
import subprocess
from gtkeasyapp.preffile import *
from gtkeasyapp.easyprocess import *
from gtkeasyapp.log import *

##Constants
PLATFORM_UNKNOWN = 0
PLATFORM_MAEMO = 1
PLATFORM_LINUX = 2
PLATFORM_WINDOWS = 3

##Easy window Exception class
class EasyWindowException(Exception):
	def __init__(self, message):
		Exception.__init__(self, message)
		
		

##Hildon window class that implements some useful features like:
## - preferences file (using PrefDialog classes)
## - application sound (using gstreamer)
## - Hildon infos messages
## - Needed UI application method (menu...)
## - Platform detection (windows, linux, maemo)
class EasyWindow():
	##Constructor
	def __init__(self, appTitle, prefFilePath='', logFilePath='', canHibernate=True):
		#set members
		self._platform = self.__detectPlatform()
		self._prefFile = None
		if len(prefFilePath)>0:
			self._prefFile = self.__loadPreferencesFile(prefFilePath)
		self._player = None
		self._mainContainer = None
		self.__vbox = None
		#self._program = None
		self._processes = {}
		if self._platform==PLATFORM_MAEMO:
			import hildon
			#self._program = hildon.Program()
			#self._program.__init__()
			
		#init process name
		if self._platform!=PLATFORM_WINDOWS:
			gobject.set_application_name(appTitle)
			
		#init log
		self.__initLog(self._prefFile, logFilePath)
	
		#init window
		if self._platform==PLATFORM_MAEMO:
			self._window = hildon.Window()
			#self._program.add_window(self._window)
			#self._program.set_can_hibernate(canHibernate)
		else:
			self._window = gtk.Window()
			self._window.set_position(gtk.WIN_POS_CENTER)
		self._window.set_title(appTitle)
		self._window.connect("destroy", self._quit)
		self._window.set_default_size(800, 500)
		self.__addPlaftormContainer()
	
		#create menu
		menu = self._createMenu(self._window, self._prefFile)
		if menu!=None:
			if self._platform==PLATFORM_WINDOWS:
				menus_bar = gtk.MenuBar()
				self.__vbox.pack_end(menus_bar, False, False)
				root_item = gtk.MenuItem(appTitle)
				root_item.set_submenu(menu)
				menus_bar.append(root_item)
			else:
				self._window.set_menu(menu)
			
		#check version
		newVersion = self.__checkVersion()
	
		#create widgets
		self._createWidgets(self._window, self._prefFile)
	
		#show ui
		self._window.show_all()
		
		#create processes
		self._createProcesses(self._prefFile)
		self.__startProcesses()
		
		#startup treatment (like open splash screen...)
		gtk.gdk.threads_enter()
		self._startupTreatment(self._window, self._prefFile, newVersion)
		gtk.gdk.threads_leave()
		
		#launch main gtk loop
		gtk.gdk.threads_enter()
		gtk.main()
		gtk.gdk.threads_leave()
		
	
	##Detect current platform
	def __detectPlatform(self):
		#detect plaftform
		if platform.platform().lower().find('arm')!=-1:
			return PLATFORM_MAEMO
		elif platform.platform().lower().find('linux')!=-1:
			return PLATFORM_LINUX
		elif platform.platform().lower().find('windows')!=-1:
			return PLATFORM_WINDOWS
		else:
			return PLATFORM_UNKNOWN
		
	
	##Add platform  container
	def __addPlaftormContainer(self):
		if self._platform==PLATFORM_WINDOWS:
			#add vbox container before main container in WIN case cause Hildon add default one in Maemo platform
			self.__vbox = gtk.VBox()
			self._window.add(self.__vbox)
			
		#create and add main container
		self._mainContainer = self._createMainContainer(self._window, self._prefFile)
		if self._platform==PLATFORM_MAEMO:
			self._window.add(self._mainContainer)
		else:
			self.__vbox.pack_start(self._mainContainer, True, True)
	
	
	##Create main container
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	##@return main GTK container
	def _createMainContainer(self, window, prefFile):
		raise EasyWindowException('_createMainContainer() must be implemented')

		
	##Window object accessor
	def _getWindow(self):
		return self._window

	
	##PrefFile object accessor
	def _getPrefFile(self):
		return self._prefFile
		
		
	##Main container accessor
	def _getMainContainer(self):
		return self._mainContainer
		
	
	##init log
	##@param prefFile : preferences file
	##@param logFile : log file
	def __initLog(self, prefFile, logFile):
		#get debug mode
		Log.MODE = self._getLogMode(prefFile, logFile)
		
		if Log.MODE==Log.MODE_FILE:
			if isinstance(logFile, str) and len(logFile)>0:
				#file
				Log.LOG_FILENAME = logFile
				Log.LOG_MAX_SIZE = 250000 #250kb
				Log.LOG_MAX_ROTATE = 2 #max 500kb of log file
			else:
				#problem with log file name, log disabled
				print 'Log filename is not correct "%s", log disabled' % logFile
				Log.MODE = Log.MODE_NONE

			
	##get log mode
	def _getLogMode(self, prefFile, logFile):
		raise EasyWindowException('_getLogMode() must be implemented')
		
		
	##Get old version (version during last launch, usualy read from preferences file)
	##@return None if no version or error occured
	def _getOldVersion(self, prefFile=None):
		raise EasyWindowException('_getOldVersion() must be implemented')

		
	##Get new version (usualy read from application global variable)
	##@return None if no version or error occured
	def _getNewVersion(self, prefFile=None):
		raise EasyWindowException('_getNewVersion() must be implemented')
		
	
	##Load preferences file
	def __loadPreferencesFile(self, filePath):
		#init
		continu = True
		test = 0
		prefFile = None
	
		while continu:
			#check if file exists
			if os.path.exists(filePath):
				#file exists, try to load it
				try:
					#increase test counter
					test += 1
					
					#create PrefFile object
					prefFile = PrefFile(filePath)
					
					#completed successfully
					continu = False
					
				except Exception, e:
					#unable to load preffile, create default one
					self._createDefaultPreferencesFile(filePath)
					
					#if after 3 tests we are unable to create prefile, exit with error
					if test>3:
						sys.exit('Unable to read or create preferences file "%s"' % (filePath))
			else:
				#file doesn't exists, create default one
				self._createDefaultPreferencesFile(filePath)
		
		return prefFile
		
	
	##Create new preferences file
	def _createDefaultPreferencesFile(self, filePath):
		raise EasyWindowException('_createDefaultPreferencesFile() must be implemented')
		
	
	##Check preferences file after new version detected (useful to add or remove item)
	##@param prefFile : PrefFile instance
	##@param filePath : preferences file path
	##@return True if pref file needs to be reloaded
	def _checkPreferencesFile(self, prefFile, filePath):
		raise EasyWindowException('_checkPreferencesFile() must be implemented')
		
		
	##Check version
	##If version is different, execute _newVersionDetected() method
	##@return True if new version detected
	def __checkVersion(self):
		newVersion = self._getNewVersion(self._prefFile)
		if self._getOldVersion(self._prefFile)!=newVersion:
			#new version detected, check preferences file
			if self._checkPreferencesFile(self._prefFile, self._prefFile.getPrefFilePath()):
				#need to reload pref file
				self._prefFile = self.__loadPreferencesFile(self._prefFile.getPrefFilePath())
				
			#update pref version
			self._updateVersion(self._prefFile, newVersion)
			
			#new version
			return True
		
		#not new version
		return False
		
	##Update version number in preferences file
	##@param prefFile : PrefFile instance
	##@param version : version to save into file
	def _updateVersion(self, prefFile, version):
		raise EasyWindowException('_updateVersion() must be implemented')

	##Create application menu
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	def _createMenu(self, window, prefFile):
		raise EasyWindowException('_createMenu() must be implemented')
		
		
	##Create widgets
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	def _createWidgets(self, window, prefFile):
		raise EasyWindowException('_createWidgets() must be implemented')
		
		
	##Startup treatment (like open splash screen...)
	##@param window : main gtk window
	##@param prefFile : PrefFile object
	##@param newVersion : True if it's a new version
	def _startupTreatment(self, window, prefFile, newVersion):
		raise EasyWindowException('_startupTreatment() must be implemented')
		
	
	##Play specified sound
	##@param filePath : path file of the sound
	def _playSound(self, filePath):
		#use madplay to play sound via cmdline
		if os.path.exists(filePath):
			cmd = '/usr/bin/madplay "%s"' % (filePath)
			try:
				subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
			except Exception, e:
				print 'EasyWindow::_playSound : unable to play sound "%s" [%s]' % (cmd, str(e))
			
		
	##Quit callback
	def _quit(self, widget, data=None):
		#before quit callback
		self._beforeQuit()
		
		#stop processes
		self.__stopProcesses()
		
		#stop gtk main loop
		gtk.main_quit()
		
	
	##Before quit callback
	def _beforeQuit(self):
		raise EasyWindowException('_beforeQuit() must be implemented')
		
		
	##Bring window to front
	def bringToFront(self):
		self._window.present()
		
		
	##Return True if window is currently activated by the window manager
	##Works only in MAEMO platform
	def isActivated():
		if self._program!=None:
			return self._program.get_is_topmost()
		else:
			return True
			
	
	##Create processes
	##@param prefFile : PrefFile object
	def _createProcesses(self, prefFile):
		raise EasyWindowException('_createProcesses() must be implemented')
	
	
	##Start processes
	def __startProcesses(self):
		for process in self._processes:
			if not self._processes[process].isAlive() and not self._processes[process].started():
				self._processes[process].start()
		
			
	##Stop processes
	def __stopProcesses(self):
		#stop processes
		for process in self._processes:
			self._processes[process].stop()
		
		#empty processes container
		self._processes.clear()
	

	##Restart all processes
	##@param timeout : delay to wait before restarting process
	def _restartProcesses(self, timeout=2.0):
		#stop all processes
		self.__stopProcesses()
		
		#wait until and of timer and restart processes
		threading.Timer(timeout, self.__restartProcesses).start()
	
	
	##Create and restart processes
	def __restartProcesses(self):
		#create processes
		self._createProcesses(self._prefFile)
		
		#start them
		self.__startProcesses()
	
	
	##Add process
	##@param processName : name of the process to add
	##@param easyProcess : EasyProcess instance to add
	##@return True if process added successfully
	def _addProcess(self, processName, easyProcess):
		if easyProcess!=None and isinstance(easyProcess, EasyProcess):
			if not self._processes.has_key(processName):
				self._processes[processName] = easyProcess
				return True
			else:
				raise EasyWindowException('_addProcess() : "%s" already exists!' % (processName) )
				return False
		else:
			raise EasyWindowException('_addProcess() : Process added must be EasyProcess instance')
			return False
	
	##Get process
	##@param processName : name of the process to get
	##@return EasyProcess instance or None if process name not found
	def getProcess(self, processName):
		if self._processes.has_key(processName):
			return self._processes[processName]
		else:
			return None
	
	
	##Update UI
	##This method is used to centralize UI updates
	##This method can call "protected" _updateUi() to securely update UI
	def updateUi(self, *args):
		raise EasyWindowException('updateUi() must be implemented')
	
	
	##Update UI using GTK thread protection
	##Must be call in inherited method updateUi()
	def _updateUi(self, callback, args):
		gobject.idle_add(callback, args)
		
		